(***********************************************************************)
(*                                                                     *)
(*                           Htmlc                                     *)
(*                                                                     *)
(*                    Pierre Weis, INRIA Rocquencourt                  *)
(*                                                                     *)
(*  Copyright 1997-2007,                                               *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  Distributed only by permission.                                    *)
(*                                                                     *)
(***********************************************************************)

(* $Id: debug.mli,v 1.6 2015/11/18 01:07:34 weis Exp $ *)

(** {3 Some simple generic tools for debugging} *)

(** This module helps printing debugging information to standard error.
  Printing is done via module [Format] and module [Debug] ensures that a
  vertical box at level 0 is always open (this way the simple space line
  breaking indication ([@ ]) always leads to a new line).
*)

val get_debug : unit -> bool
(** Return the current debugging mode. *)
;;

val set_debug : unit -> unit
(** Turn debugging on. *)
;;

val failwith : string -> 'a
(** When debugging is on, [Debug.failwith] prints its arguments on [stderr],
  then raises a [Failure] exception. *)
;;

val debug : ('a, Format.formatter, unit) format -> 'a
(** When debugging is on, [Debug.debug] prints its arguments on
  [Format.err_formatter]. *)
;;

val fdebug : (Format.formatter -> unit) -> unit
(** When debugging is on, [Debug.fdebug f] prints debugging message from
  function [f] by evaluating [f Format.err_formatter]. *)
;;

val message : string -> unit
(** When debugging is on, [Debug.message] outputs the given message on
  [Format.err_formatter]. *)
;;

val warning : string -> unit
(** [Debug.warning] unconditionally outputs a warning on [Format.err_formatter]. *)
;;

val pause : string -> unit
(** When debugging is on, [Debug.pause] outputs outputs the given message on
    [Format.err_formatter], then stops the program.
    Press return to go on. *)
;;

val close : unit -> unit
(** Flush the error formatter at end of debugging. *)
;;
val flush : unit -> unit
(** Flush the error formatter and reopen a vertical box at level 0 to
  print further debugging material. *)
;;
