(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2009-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.2+dev0 generated code. *)

let printer_size ctx ppf = function
  | u_s -> Umatrix_universe_print.printer_size ctx ppf u_s
;;

let printer_element ctx ppf = function
  | u_e -> Umatrix_universe_print.printer_element ctx ppf u_e
;;

let printer_uindex ctx ppf = function
  | u_i -> Umatrix_universe_print.printer_index ctx ppf u_i
;;

let printer_index ctx ppf = function
  | e_u_i ->
    Uindex_universe_print.printer_index printer_element ctx ppf e_u_i
;;

let printer_t ctx ppf = function
  | {
      Umatrix_nonempty_set.universe = u_t;
      Umatrix_nonempty_set.uindices = e_u_t;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>universe =";
    Umatrix_universe_print.printer_t Og_precedence.Ck_record_field ppf u_t;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>uindices =";
    Uindex_universe_print.printer_t printer_element
      Og_precedence.Ck_record_field
      ppf
      e_u_t;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_nonempty_subset ctx ppf = function
  | {
      Umatrix_nonempty_set.support = t;
      Umatrix_nonempty_set.indices = i_a;
      Umatrix_nonempty_set.nonempty_set = t0;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>support =";
    printer_t Og_precedence.Ck_record_field ppf t;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>indices =";
    Og_printer.printer_Array printer_index
      Og_precedence.Ck_record_field
      ppf
      i_a;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>nonempty_set =";
    printer_t Og_precedence.Ck_record_field ppf t0;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_dichotomy ctx ppf = function
  | (n, n0) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_nonempty_subset Og_precedence.Ck_tuple_item ppf n;
    Format.fprintf ppf "%,,@ ";
    printer_nonempty_subset Og_precedence.Ck_tuple_item ppf n0;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

(* Defining top printers. *)

let print_size ppf = printer_size Og_precedence.Ck_toplevel ppf
;;

let print_element ppf = printer_element Og_precedence.Ck_toplevel ppf
;;

let print_uindex ppf = printer_uindex Og_precedence.Ck_toplevel ppf
;;

let print_index ppf = printer_index Og_precedence.Ck_toplevel ppf
;;

let print_t ppf = printer_t Og_precedence.Ck_toplevel ppf
;;

let print_nonempty_subset ppf =
  printer_nonempty_subset Og_precedence.Ck_toplevel ppf
;;

let print_dichotomy ppf = printer_dichotomy Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.2+dev0 generated code. *)
