(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2009-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of OCamlGen, 4.2+dev0 generated code. *)

let printer_start_mode _ctx ppf = function
  | History_types.Cold_start ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Cold_start"
  | History_types.Warm_start ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Warm_start"
;;

let printer_history_file_name ctx ppf = function
  | u_t -> Ufilename_print.printer_t ctx ppf u_t
;;

let printer_write_directory_name ctx ppf = function
  | u_t -> Ufilename_print.printer_t ctx ppf u_t
;;

let printer_history_step_mode _ctx ppf = function
  | History_types.Last ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Last"
  | History_types.Full ->
    if _ctx = Og_precedence.Ck_argument ||
       _ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,Full"
;;

let printer_base_name ctx ppf = function
  | u_t -> Ufilename_print.printer_t ctx ppf u_t
;;

let printer_checkpointing ctx ppf = function
  | b_o -> Og_printer.printer_Option printer_base_name ctx ppf b_o
;;

let printer_history_mode ctx ppf = function
  | (h, c) ->
    if ctx = Og_precedence.Ck_record_field then Format.fprintf ppf "%,@ "
      else
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%, (@," else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@[<hv 2>(@,";
    Format.fprintf ppf "%,@[<hv 0>";
    printer_history_step_mode Og_precedence.Ck_tuple_item ppf h;
    Format.fprintf ppf "%,,@ ";
    printer_checkpointing Og_precedence.Ck_tuple_item ppf c;
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument then Format.fprintf ppf "%,@;<0 -2>)"
      else
    if ctx = Og_precedence.Ck_tuple_item then
      Format.fprintf ppf "%,@;<0 -2>)@]"
;;

let printer_state_number ctx ppf = function
  | u_t -> Unatural_print.printer_t ctx ppf u_t
;;

let printer_h_state pp_a ctx ppf = function
  | { History_types.state_number = s; History_types.state = a; } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>state_number =";
    printer_state_number Og_precedence.Ck_record_field ppf s;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>state =";
    pp_a Og_precedence.Ck_record_field ppf a; Format.fprintf ppf "%,;@]";
    Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

let printer_h_list pp_a ctx ppf = function
  | a_h_u_l ->
    Ulist1_print.printer_list1 (printer_h_state pp_a) ctx ppf a_h_u_l
;;

let printer_t pp_a pp_b ctx ppf = function
  | {
      History_types.configuration = a;
      History_types.h_mode = h;
      History_types.h_list = b_h;
    } ->
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%, {@ " else
    Format.fprintf ppf "%,@[<hv 2>{@ "; Format.fprintf ppf "%,@[<hv 0>";
    Format.fprintf ppf "%,@[<hv 2>configuration =";
    pp_a Og_precedence.Ck_record_field ppf a; Format.fprintf ppf "%,;@]";
    Format.fprintf ppf "%,@ "; Format.fprintf ppf "%,@[<hv 2>h_mode =";
    printer_history_mode Og_precedence.Ck_record_field ppf h;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@ ";
    Format.fprintf ppf "%,@[<hv 2>h_list =";
    printer_h_list pp_b Og_precedence.Ck_record_field ppf b_h;
    Format.fprintf ppf "%,;@]"; Format.fprintf ppf "%,@]";
    if ctx = Og_precedence.Ck_argument || ctx = Og_precedence.Ck_record_field
      then Format.fprintf ppf "%,@;<1 -2>}" else
    Format.fprintf ppf "%,@;<1 -2>}@]"
;;

(* Defining top printers. *)

let print_start_mode ppf = printer_start_mode Og_precedence.Ck_toplevel ppf
;;

let print_history_file_name ppf =
  printer_history_file_name Og_precedence.Ck_toplevel ppf
;;

let print_write_directory_name ppf =
  printer_write_directory_name Og_precedence.Ck_toplevel ppf
;;

let print_history_step_mode ppf =
  printer_history_step_mode Og_precedence.Ck_toplevel ppf
;;

let print_base_name ppf = printer_base_name Og_precedence.Ck_toplevel ppf
;;

let print_checkpointing ppf =
  printer_checkpointing Og_precedence.Ck_toplevel ppf
;;

let print_history_mode ppf =
  printer_history_mode Og_precedence.Ck_toplevel ppf
;;

let print_state_number ppf =
  printer_state_number Og_precedence.Ck_toplevel ppf
;;

let print_h_state pp_a ppf =
  printer_h_state pp_a Og_precedence.Ck_toplevel ppf
;;

let print_h_list pp_a ppf = printer_h_list pp_a Og_precedence.Ck_toplevel ppf
;;

let print_t pp_a pp_b ppf = printer_t pp_a pp_b Og_precedence.Ck_toplevel ppf
;;

(* End of OCamlGen, 4.2+dev0 generated code. *)
