(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2009-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: monitor.ml,v 1.11 2017/05/31 16:38:15 fclement Exp $ *)

open Segmentation_types;;
open Segmentation_types_print;;

let interruption ppf =
  Format.fprintf ppf "Hit <Enter> to continue...@.";
  ignore (input_line stdin);
;;

type 'a sub_zone_report = zone_number * Upervasives.length * 'a;;
type 'a zone_splitting_report =
  zone_number * 'a sub_zone_report * 'a sub_zone_report
;;

let print_zone_number ppf zn =
  Format.fprintf ppf "zone #%a" print_zone_number zn
;;

let print_sub_zone_report printer_scalar ppf (zn, card, param) =
  Format.fprintf ppf "  -> %a, card = %a, param = %a"
    print_zone_number zn
    Upervasives_print.print_length card
    (printer_scalar Og_precedence.Ck_toplevel) param
;;

let printer_zone_splitting_report printer_scalar _ctx ppf (zn, szf, szt) =
  Format.fprintf ppf "\nsplit %a@ %a@ %a@ "
    print_zone_number zn
    (print_sub_zone_report printer_scalar) szf
    (print_sub_zone_report printer_scalar) szt
;; 

let print_iteration_report print_scalar ppf (i, j) =
  Format.fprintf ppf "@[iter = %a, j = %a@]@."
    Unatural_print.print_t i
    print_scalar j
;;

type memory_unit =
  | B | KiB | MiB | GiB | TiB | PiB | EiB | ZiB | YiB
;;

let string_of_memory_unit = function
  | B -> "bytes"
  | KiB -> "kibibytes"
  | MiB -> "mebibytes"
  | GiB -> "gibibytes"
  | TiB -> "tebibytes"
  | PiB -> "pebibytes"
  | EiB -> "exbibytes"
  | ZiB -> "zebibytes"
  | YiB -> "yobibytes"
;;

let total_allocated_memory () =
  let tam = Gc.allocated_bytes () in
  let rec loop ((x, _) as accu) = function
    | [] -> accu
    | mu :: l -> if x < 1024. then accu else loop (x /. 1024., mu) l in
  let (x, mu) = loop (tam, B) [ KiB; MiB; GiB; TiB; PiB; EiB; ZiB; YiB; ] in
  (x, string_of_memory_unit mu)
;;
