(***********************************************************************)
(*                                                                     *)
(*                              Ref-image                              *)
(*                                                                     *)
(*                  Image Segmentation by Refinement                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2009-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ustring_command_option.ml,v 1.9 2017/05/31 16:38:16 fclement Exp $ *)

module Cmd_opt = Ucommand_option;;

open Ustring_command_option_spec;;

module Make : MAKER =
  functor (S : STRUCTURE) ->
struct
  include S;;
  
  let print_quoted _ctx ppf x = Format.fprintf ppf "%S" (to_string x);;
  
  let { Uaref.get = get; set = set; } = Uaref.make_aref_single name default;;
  let set_from_string = Upervasives.compose set of_string;;
  let print_all ppf =
    Og_printer.printer_List print_quoted Og_precedence.Ck_toplevel ppf all
  ;;
  let print_default ppf = print_quoted Og_precedence.Ck_toplevel ppf (get ());;
  
  let buf = Buffer.create 1024;;
  let ppf = Format.formatter_of_buffer buf;;
  Format.fprintf ppf
    "@[<v 8><str>@ \
    @[Set %s to@ <str>.@]@ \
    @[Valid %s are:@ %t.@]@ \
    Default %s is %t.@]"
    name
    plural_name print_all
    name print_default
  ;;
  let doc = Buffer.contents buf;;
  Buffer.clear buf;;
  
  let lok = Uoption_key.long_option_key_of_string name;;
  
  let rco =
    {
     Cmd_opt.key = lok;
     spec = Arg.String set_from_string;
     doc = doc;
    }
  ;;
end
;;
