(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: uarray1_with_fortran_layout.ml,v 1.18 2017/05/31 15:24:06 fclement Exp $ *)

(* More operations on one-dimensional bigarrays with Fortran layout. *)

let qual = Upervasives.qualify "Uarray1_with_fortran_layout";;
let invalid_arg = Upervasives.invalid_arg_qual qual;;

open Bigarray;;

let length = Array1.dim
and get = Array1.get
and set = Array1.set;;

let init kind n f =
  let a = Array1.create kind fortran_layout n in
  for i = 1 to n do set a i (f i) done;
  a;;

let make kind n x =
  let a = Array1.create kind fortran_layout n in
  Array1.fill a x;
  a;;

(* Iterators. *)

let iter f a =
  let n = length a in
  for i = 1 to n do f (get a i) done;;

let map f a =
  let n = length a in
  let b = Array1.create (Array1.kind a) fortran_layout n in
  for i = 1 to n do set b i (f (get a i)) done;
  b;;

let fold_left f x a =
  let n = length a in
  let r = ref x in
  for i = 1 to n do r := f !r (get a i) done;
  !r;;

(* Iterators on two bigarrays. *)

let same_length fun_name a1 a2 =
  let n = length a1 in
  if n = length a2 then n else
  invalid_arg fun_name "different lengths"
;;

let fold_left2 f x a1 a2 =
  let n = same_length "fold_left2" a1 a2 in
  let r = ref x in
  for i = 1 to n do r := f !r (get a1 i) (get a2 i) done;
  !r;;
