(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udelayed_dgraph.mli,v 1.15 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 Delayed dynamic graphs}

 Direct access data structure. *)

type arrow_number = Udelayed_dgraph_types.arrow_number;;

type 'b delayed_image = 'b Udelayed_dgraph_types.delayed_image =
  | Not_yet_computed
  | Image of 'b
(** Type ['a delayed_image] is equivalent to ['a option]. *)
;;

type ('a, 'b) delayed_arrow = ('a, 'b) Udelayed_dgraph_types.delayed_arrow
(** The ``option'' indicates that the image may not yet be computed. *)
;;

type ('a, 'b, 'c) t = ('a, 'b, 'c) Udelayed_dgraph_types.t = private
  {
   f : 'c -> arrow_number -> 'a -> 'b;
   graph : ('a, 'b delayed_image) Udynamic_graph.t;
   to_do : arrow_number list;
  }
and ('a, 'b, 'c) delayed_dgraph = ('a, 'b, 'c) t
(** First argument of function [f] of type ['c] may represent any additional
 argument parameter, other than that of type ['a]. If useless, simply put [unit]
 The todo list gathers the indices of the not-yet-computed images. *)
;;

(** {6 Construction functions} *)

val make :
  ('c -> arrow_number -> 'a -> 'b) -> arrow_number -> 'a -> ('a, 'b, 'c) t
;;

val set_next : ('a, 'b, 'c) t -> 'a -> ('a, 'b, 'c) t;;
val set : ('a, 'b, 'c) t -> arrow_number -> 'a -> ('a, 'b, 'c) t;;

val of_values :
  ('c -> arrow_number -> 'a -> 'b) -> 'a array -> ('a, 'b, 'c) t
(** The resulting delayed dynamic graph is fully delayed: all images in the
 arrows are [Not_yet_computed]. *)
;;

val do_it : 'c -> ('a, 'b, 'c) t -> ('a, 'b, 'c) t;;

(** {6 Operations on delayed dynamic graphs} *)

val get : ('a, 'b, 'c) t -> arrow_number -> ('a, 'b) delayed_arrow;;

val same : ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> bool;;
val equal_values :
  ('a -> 'a -> bool) -> ('a, 'b, 'c) t -> ('a, 'b, 'c) t -> bool
;;

val to_array : ('a, 'b, 'c) t -> ('a, 'b) Udynamic_graph.arrow array
(** Raise [Invalid_arg] when the to_do list is nonempty. *)
;;

(** {6 Iterators} *)

val iter :
  (arrow_number -> ('a, 'b) delayed_arrow -> unit) -> ('a, 'b, 'c) t -> unit
(** Raise [Invalid_arg] when the to_do list is nonempty. *)
;;

val fold_left :
  ('accu -> arrow_number -> ('a, 'b) delayed_arrow -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
(** Raise [Invalid_arg] when the to_do list is nonempty. *)
;;

val iter_to_do : (arrow_number -> 'a -> unit) -> ('a, 'b, 'c) t -> unit
(** Iterates on the to_do list only. *)
;;

val iter_value : (arrow_number -> 'a -> unit) -> ('a, 'b, 'c) t -> unit;;
val fold_left_value :
  ('accu -> arrow_number -> 'a -> 'accu) ->
  'accu -> ('a, 'b, 'c) t -> 'accu
(** Only consider the value field of the delayed arrows. *)
;;
