(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: udynamic_graph.ml,v 1.14 2017/05/31 15:24:06 fclement Exp $ *)

(* Dynamic graphs. *)

module A = Uarray;;
module Sda = Ustamped_darray;;

type length = Sda.length;;
type arrow_number = Sda.index;;

type ('a, 'b) arrow =
  {
   value : 'a;
   image : 'b;
  }
;;

let get_value ar = ar.value;;

type ('a, 'b) t = ('a, 'b) arrow Sda.t
and ('a, 'b) dynamic_graph = ('a, 'b) t
;;

let make = Sda.make;;

let get = Sda.get;;
let set = Sda.set;;

let next_unset dg = dg.Sda.next_unset;;
let set_next dg = set dg (next_unset dg);;

let same = Sda.same;;

let equal_values equal_value dg1 dg2 =
  let equal_value ar1 ar2 = equal_value ar1.value ar2.value in
  Sda.equal_items equal_value dg1 dg2
;;

(* Converters. *)

let to_array = Sda.to_array;;

let to_values dg = A.map get_value (to_array dg);;

let arrow f x = { value = x; image = f x; };;

let of_values f a =
  let aa = A.map (arrow f) a in
  Sda.of_array aa
;;

(* Iterators. *)

let iter = Sda.iter;;
let map = Sda.map;;
let fold_left = Sda.fold_left;;
