(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: ugraph.ml,v 1.12 2017/05/31 15:24:06 fclement Exp $ *)

(* Graphs of functions. *)

module A = Uarray;;
module Gt = Ugraph_types;;
module Nset = Unonempty_set;;

let qual = Upervasives.qualify "Ugraph";;

let invalid_arg = Upervasives.invalid_arg_qual qual;;

type 'a index = 'a Gt.index
and 'a uindex = 'a Gt.uindex
;;

type 'a domain = 'a Gt.domain
and 'a sub_domain = 'a Nset.nonempty_subset
;;

type 'b images = 'b Gt.images =
  | Constant of 'b
  | Images of 'b array
;;

type ('a, 'b) t = ('a, 'b) Gt.t = private
  {
   domain : 'a domain;
   images : 'b images;
  }
and ('a, 'b) graph = ('a, 'b) t
;;

let same g1 g2 = g1 == g2;;

(* Construction functions. *)

let constant = Gt.constant
and from_array = Gt.from_array
and from_fun = Gt.from_fun
and of_matrix = Gt.of_matrix
;;

(* Functions on graphs. *)

let image g =
  match g.images with
  | Constant y -> (fun _index -> y)
  | Images ims -> (fun index -> A.get ims (Uindex.to_nat index))
;;

let restriction
    ({ domain = dom; images = images; }  as g)
    { Nset.support = sup;
      indices = indices;
      nonempty_set = sub_dom; } =
  if sup != dom then
    invalid_arg "restriction" "sub-domain is not a subset of domain" else
  match images with
  | Constant y -> constant sub_dom y
  | Images _ -> from_array sub_dom (A.map (image g) indices)
;;

(* Iterators. *)

let map_images = Gt.map_images;;
