(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: umatrix_types.mli,v 1.10 2017/05/31 15:24:06 fclement Exp $ *)

(** {3 The private type of matrices} *)

type 'a t = private 'a array array
and 'a matrix = 'a t
;;

type length = Uarray.length
and index = Uarray.index

and line_number = length
and column_number = length
and line_index = index
;;

type size = line_number * column_number;;

val empty : 'a t;;

val to_t : 'a array array -> 'a t;;
external of_t : 'a t -> 'a array array = "%identity";;

val set_line : 'a t -> line_index -> 'a array -> unit;;

val size : 'a t -> size;;
val dim_i : 'a t -> line_number;;
val dim_j : 'a t -> column_number;;

val check_bound :
  Upervasives.message -> Upervasives.function_name -> index -> length -> unit
;;
val check_line_bound :
  Upervasives.function_name -> index -> length -> unit
;;
val check_array_length :
  Upervasives.function_name -> 'a array -> length -> unit
;;
