(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upositive_rational.ml,v 1.11 2017/05/31 15:24:06 fclement Exp $ *)

(* Positive rational numbers. *)

module Nat = Unatural;;
module Prt = Upositive_rational_types;;

type t = Prt.t = private
  {
   numerator : Unatural.t;
   denominator : Unatural.t;
  }
and positive_rational = t
;;

let of_nat = Prt.to_t;;

let _1_ = of_nat Nat._1_ Nat._1_;;

let scale { numerator = p; denominator = q; } n =
  Nat.div (Nat.mul n p) q
;;

(* Comparison. *)

let compare
    { numerator = p1; denominator = q1; }
    { numerator = p2; denominator = q2; } =
  Pervasives.compare (Nat.mul p1 q2) (Nat.mul p2 q1)
;;

let comparison_lift cmp r1 r2 = cmp (compare r1 r2) 0;;

let equal = comparison_lift ( = );;
let different = comparison_lift ( <> );;
let smaller = comparison_lift ( < );;
let greater = comparison_lift ( > );;
let smaller_or_equal = comparison_lift ( <= );;
let greater_or_equal = comparison_lift ( >= );;

let min r1 r2 = if smaller_or_equal r1 r2 then r1 else r2;;
let max r1 r2 = if greater_or_equal r1 r2 then r1 else r2;;
