(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: upositive_rational_range.ml,v 1.13 2017/05/31 15:24:06 fclement Exp $ *)

(* Operations on positive rational number ranges. *)

module Rat = Upositive_rational;;
module Prrt = Upositive_rational_range_types;;

let qual = Upervasives.qualify "Upositive_rational_range";;
let failwith = Upervasives.failwith_qual qual;;

type nonempty_range = Prrt.nonempty_range = private
  {
   left : Rat.t;
   right : Rat.t;
  }
and t = Prrt.t =
  | Empty
  | Nonempty of nonempty_range
and positive_rational_range = t
;;

let of_rat r1 r2 =
  try Nonempty (Prrt.to_nonempty_range r1 r2) with
  | Failure _ -> Empty
;;

let intersection rg1 rg2 =
  match (rg1, rg2) with
  | (Empty, _) | (_, Empty) -> Empty
  | (Nonempty { left = l1; right = r1; },
     Nonempty { left = l2; right = r2; }) ->
       of_rat (Rat.max l1 l2) (Rat.min r1 r2)
;;

let project rg x =
  match rg with
  | Empty -> failwith "project" "empty range"
  | Nonempty { left = l; right = r; } ->
      if Rat.smaller_or_equal l x && Rat.smaller_or_equal x r then x else
      if Rat.smaller r x then r else
      l
;;
