(***********************************************************************)
(*                                                                     *)
(*                                Ulib                                 *)
(*                                                                     *)
(*                   OCaml user's additional library                   *)
(*                                                                     *)
(*                       EPI Serena, INRIA Paris                       *)
(*                                                                     *)
(*  Copyright (c) 2004-2017 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: utype_digest.ml,v 1.16 2017/05/31 15:24:06 fclement Exp $ *)

(* Runtime identification of types. *)

type 'a t = Digest.t
and 'a type_digest = 'a t
;;

type ('a, 'b) lifter = 'a t -> 'b t;;
type ('a, 'b, 'c) lifter2 = 'a t -> 'b t -> 'c t;;
type ('a, 'b, 'c, 'd) lifter3 = 'a t -> 'b t -> 'c t -> 'd t;;
type ('a, 'b, 'c, 'd, 'e) lifter4 = 'a t -> 'b t -> 'c t -> 'd t -> 'e t;;
type ('a, 'b, 'c, 'd, 'e, 'f, 'g) lifter6 =
  'a t -> 'b t -> 'c t -> 'd t -> 'e t -> 'f t -> 'g t
;;

type type_name = string;;

let of_string = Digest.string;;

let gen_succ_digest gen m_md5 typ a_md5 = gen m_md5 (typ ^ a_md5);;

let gen_digest m_md5 typ = of_string (m_md5 ^ typ);;

let gen_unary_digest = gen_succ_digest gen_digest;;
let gen_binary_digest = gen_succ_digest gen_unary_digest;;
let gen_ternary_digest = gen_succ_digest gen_binary_digest;;
let gen_quaternary_digest = gen_succ_digest gen_ternary_digest;;
let gen_quinternary_digest = gen_succ_digest gen_quaternary_digest;;
let gen_sexternary_digest = gen_succ_digest gen_quinternary_digest;;

(* Builtin types from OCaml system. *)

let builtin_digest = of_string "";;

let gen_builtin_digest = gen_digest builtin_digest;;

let exception_digest = gen_builtin_digest "exn";;

let unit_digest = gen_builtin_digest "unit"
and bool_digest = gen_builtin_digest "bool"
and char_digest = gen_builtin_digest "char"
and int_digest = gen_builtin_digest "int"
and int32_digest = gen_builtin_digest "int32"
and int64_digest = gen_builtin_digest "int64"
and nativeint_digest = gen_builtin_digest "nativeint"
and float_digest = gen_builtin_digest "float"
and string_digest = gen_builtin_digest "string"
;;

let gen_builtin_unary_digest = gen_unary_digest builtin_digest
and gen_builtin_binary_digest = gen_binary_digest builtin_digest
and gen_builtin_ternary_digest = gen_ternary_digest builtin_digest
;;

let option_digest = gen_builtin_unary_digest "option"
and list_digest = gen_builtin_unary_digest "list"
and array_digest = gen_builtin_unary_digest "array"
;;

let arrow_digest = gen_builtin_binary_digest "->";;

let couple_digest = gen_builtin_binary_digest "*"
and triple_digest = gen_builtin_ternary_digest "*"
;;

(* Types from Pervasives. *)

let pervasives_digest = of_string "Pervasives";;

let gen_pervasives_digest = gen_digest pervasives_digest
and gen_pervasives_unary_digest = gen_unary_digest pervasives_digest
and gen_pervasives_sexternary_digest =
  gen_sexternary_digest pervasives_digest
;;

let fpclass_digest = gen_pervasives_digest "fpclass"
and in_channel_digest = gen_pervasives_digest "in_channel"
and out_channel_digest = gen_pervasives_digest "out_channel"
and open_flag_digest = gen_pervasives_digest "open_flag"
;;

let ref_digest = gen_pervasives_unary_digest "ref";;

let format6_digest = gen_pervasives_sexternary_digest "format6";;
let format4_digest a_md5 b_md5 c_md5 d_md5 =
  format6_digest a_md5 b_md5 c_md5 c_md5 c_md5 d_md5
;;
let format_digest a_md5 b_md5 c_md5 =
  format4_digest a_md5 b_md5 c_md5 c_md5
;;

(* Basic types from Ulib. *)

let list1_digest a_md5 = couple_digest a_md5 (list_digest a_md5);;
